@@ -1,3 +1,7 @@
+.onUnload <- function (libpath) {
+  library.dynam.unload("matrixStats", libpath)
+}
+
 .onAttach <- function(libname, pkgname) {
   pd <- utils::packageDescription(pkgname);
   pkgStartupMessage(pkgname, " v", pd$Version, " (",
